@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
-"""
+import sys
+if sys.version_info[:2] > (2,4): # No doctest.skip in Python 2.4
+    __doc__ = """
 A collection of utilities for canonicalizing and inspecting graphs.
 
 Among other things, they solve of the problem of deterministic bnode
@@ -43,38 +45,104 @@ Present in both::
 
     >>> def dump_nt_sorted(g):
     ...     for l in sorted(g.serialize(format='nt').splitlines()):
-    ...         if l: print l
+    ...         if l: print(l.decode('ascii'))
 
-    >>> dump_nt_sorted(in_both)
+    >>> dump_nt_sorted(in_both) #doctest: +SKIP
     <http://example.org> <http://example.org/ns#rel> <http://example.org/same> .
-    <http://example.org> <http://example.org/ns#rel> _:cb1373e1895e37293a13204e8048bdcdc7 .
-    _:cb1373e1895e37293a13204e8048bdcdc7 <http://example.org/ns#label> "Same" .
+    <http://example.org> <http://example.org/ns#rel> _:cbcaabaaba17fecbc304a64f8edee4335e .
+    _:cbcaabaaba17fecbc304a64f8edee4335e <http://example.org/ns#label> "Same" .
 
 Only in first::
 
-    >>> dump_nt_sorted(in_first)
+    >>> dump_nt_sorted(in_first) #doctest: +SKIP
     <http://example.org> <http://example.org/ns#rel> <http://example.org/a> .
-    <http://example.org> <http://example.org/ns#rel> _:cb12f880a18a57364752aaeb157f2e66bb .
-    _:cb12f880a18a57364752aaeb157f2e66bb <http://example.org/ns#label> "A" .
+    <http://example.org> <http://example.org/ns#rel> _:cb124e4c6da0579f810c0ffe4eff485bd9 .
+    _:cb124e4c6da0579f810c0ffe4eff485bd9 <http://example.org/ns#label> "A" .
 
 Only in second::
 
-    >>> dump_nt_sorted(in_second)
+    >>> dump_nt_sorted(in_second) #doctest: +SKIP
     <http://example.org> <http://example.org/ns#rel> <http://example.org/b> .
-    <http://example.org> <http://example.org/ns#rel> _:cb0a343fb77929ad37cf00a0317f06b801 .
-    _:cb0a343fb77929ad37cf00a0317f06b801 <http://example.org/ns#label> "B" .
+    <http://example.org> <http://example.org/ns#rel> _:cb558f30e21ddfc05ca53108348338ade8 .
+    _:cb558f30e21ddfc05ca53108348338ade8 <http://example.org/ns#label> "B" .
 """
+else:
+    __doc__ = """"""
+
+# ======================================================================
+# FAIL: Doctest: rdflib.compare
+# ----------------------------------------------------------------------
+# Traceback (most recent call last):
+#   File "/usr/lib/python2.7/doctest.py", line 2166, in runTest
+#     raise self.failureException(self.format_failure(new.getvalue()))
+# AssertionError: Failed doctest test for rdflib.compare
+#   File "...rdflib/rdflib/compare.py", line 1, in compare
+# 
+# ----------------------------------------------------------------------
+# File "...rdflib/rdflib/compare.py", line 48, in rdflib.compare
+# Failed example:
+#     dump_nt_sorted(in_both) #doctest +SKIP
+# Expected:
+#     <http://example.org> <http://example.org/ns#rel> <http://example.org/same> .
+#     <http://example.org> <http://example.org/ns#rel> _:cbcaabaaba17fecbc304a64f8edee4335e .
+#     _:cbcaabaaba17fecbc304a64f8edee4335e <http://example.org/ns#label> "Same" .
+# Got:
+#     <http://example.org> <http://example.org/ns#rel> <http://example.org/same> .
+# ----------------------------------------------------------------------
+# File "...rdflib/rdflib/compare.py", line 55, in rdflib.compare
+# Failed example:
+#     dump_nt_sorted(in_first) #doctest +SKIP
+# Expected:
+#     <http://example.org> <http://example.org/ns#rel> <http://example.org/a> .
+#     <http://example.org> <http://example.org/ns#rel> _:cb124e4c6da0579f810c0ffe4eff485bd9 .
+#     _:cb124e4c6da0579f810c0ffe4eff485bd9 <http://example.org/ns#label> "A" .
+# Got:
+#     <http://example.org> <http://example.org/ns#rel> <http://example.org/a> .
+#     <http://example.org> <http://example.org/ns#rel> _:cb189fca567334c3d20481a6d4035592bc .
+#     <http://example.org> <http://example.org/ns#rel> _:cbd80360ccf6ce9f9aa20dd0a4e90027e4 .
+#     _:cb65019af46ad8af18df6cbce90af81a02 <http://example.org/ns#label> "Same" .
+#     _:cba6f22538a1d3cf645d95dcc441170f24 <http://example.org/ns#label> "A" .
+# ----------------------------------------------------------------------
+# File "...rdflib/rdflib/compare.py", line 62, in rdflib.compare
+# Failed example:
+#     dump_nt_sorted(in_second) #doctest +SKIP
+# Expected:
+#     <http://example.org> <http://example.org/ns#rel> <http://example.org/b> .
+#     <http://example.org> <http://example.org/ns#rel> _:cb558f30e21ddfc05ca53108348338ade8 .
+#     _:cb558f30e21ddfc05ca53108348338ade8 <http://example.org/ns#label> "B" .
+# Got:
+#     <http://example.org> <http://example.org/ns#rel> <http://example.org/b> .
+#     <http://example.org> <http://example.org/ns#rel> _:cbd4f503467ab75a1056349c4eb47ac6ea .
+#     <http://example.org> <http://example.org/ns#rel> _:cbd6fd45be5f6f1a929dca5f11f72ccae2 .
+#     _:cb8a1b89fb2a3e9e99143f2dffbc5e0bf4 <http://example.org/ns#label> "B" .
+#     _:cbc9878f3250eee5de9cb6212906a0972f <http://example.org/ns#label> "Same" .
+# 
+# -------------------- >> begin captured logging << --------------------
+# rdflib: INFO: version: 3.3.0-dev
+# --------------------- >> end captured logging << ---------------------
+# 
+# ----------------------------------------------------------------------
+# Ran 2 tests in 0.200s
+# 
+# FAILED (failures=1)
+
 
 # TODO:
 # - Doesn't handle quads.
 # - Add warning and/or safety mechanism before working on large graphs?
 # - use this in existing Graph.isomorphic?
 
-
+__all__ = ['IsomorphicGraph', 'to_isomorphic', 'isomorphic', 'to_canonical_graph', 'graph_diff', 'similar']
+ 
 from rdflib.graph import Graph, ConjunctiveGraph, ReadOnlyGraphAggregate
 from rdflib.term import BNode
-import hashlib
-
+try:
+    import hashlib
+    md = hashlib.md5()
+except ImportError:
+    # for Python << 2.5
+    import md5
+    md = md5.new()
 
 class IsomorphicGraph(ConjunctiveGraph):
     """
@@ -130,7 +198,8 @@ class _TripleCanonicalizer(object):
                 yield term
 
     def _canonicalize(self, term, done=False):
-        return self.hashfunc(tuple(sorted(self._vhashtriples(term, done))))
+        return self.hashfunc(tuple(sorted(self._vhashtriples(term, done),
+                                                    key=_hetero_tuple_key)))
 
     def _vhashtriples(self, term, done):
         for triple in self.graph:
@@ -145,6 +214,10 @@ class _TripleCanonicalizer(object):
                 yield i
             else:
                 yield self._canonicalize(term, done=True)
+                
+def _hetero_tuple_key(x):
+    "Sort like Python 2 - by name of type, then by value. Expects tuples."
+    return tuple((type(a).__name__, a) for a in x)
 
 
 def to_isomorphic(graph):
@@ -211,12 +284,12 @@ def graph_diff(g1, g2):
 
 
 def _md5_hash(t):
-    h = hashlib.md5()
+    h = md
     for i in t:
         if isinstance(i, tuple):
-            h.update(_md5_hash(i))
+            h.update(_md5_hash(i).encode('ascii'))
         else:
-            h.update("%s" % i)
+            h.update(unicode(i).encode("utf8"))
     return h.hexdigest()
 
 
