@@ -26,13 +26,19 @@ See the `setuptools dynamic discovery of services and plugins`__ for more inform
 from rdflib.store import Store
 from rdflib.parser import Parser
 from rdflib.serializer import Serializer
-from rdflib import query
+from rdflib.query import ResultParser, ResultSerializer, Processor, Result
 from rdflib.exceptions import Error
 
+__all__ = ['register', 'get', 'plugins', 'PluginException', 'Plugin', 'PKGPlugin']
 
 entry_points = {'rdf.plugins.store': Store,
                 'rdf.plugins.serializer': Serializer,
-                'rdf.plugins.parser': Parser}
+                'rdf.plugins.parser': Parser,
+                'rdf.plugins.resultparser': ResultParser,
+                'rdf.plugins.resultserializer': ResultSerializer,
+                'rdf.plugins.queryprocessor': Processor,
+                'rdf.plugins.queryresult': Result
+                }
 
 _plugins = {}
 
@@ -132,6 +138,8 @@ register('pretty-xml', Serializer,
                 'rdflib.plugins.serializers.rdfxml', 'PrettyXMLSerializer')
 register('trix', Serializer,
                 'rdflib.plugins.serializers.trix', 'TriXSerializer')
+register("nquads", Serializer, 
+                 'rdflib.plugins.serializers.nquads', 'NQuadsSerializer')
 
 register('application/rdf+xml', Parser,
                 'rdflib.plugins.parsers.rdfxml', 'RDFXMLParser')
@@ -143,10 +151,15 @@ register('xml', Parser,
                 'rdflib.plugins.parsers.rdfxml', 'RDFXMLParser')
 register('n3', Parser, 
                 'rdflib.plugins.parsers.notation3', 'N3Parser')
+register('turtle', Parser, 
+                'rdflib.plugins.parsers.notation3', 'N3Parser')
 register('nt', Parser, 
                 'rdflib.plugins.parsers.nt', 'NTParser')
+register('nquads', Parser, 
+                'rdflib.plugins.parsers.nquads', 'NQuadsParser')
 register('trix', Parser, 
                 'rdflib.plugins.parsers.trix', 'TriXParser')
 register('rdfa', Parser, 
                 'rdflib.plugins.parsers.rdfa', 'RDFaParser')
 
+register("nquads", Parser, "rdflib.plugins.parsers.nquads", "NQuadsParser")
